==> membrane_core-0.4.0.tar: lib/membrane/element/callback_context.ex <==
defmodule Membrane.Element.CallbackContext do
  @moduledoc """
  Parent module for all contexts passed to callbacks.

  The idea of context is to provide access to commonly used information without
  holding it in elements state. Context differs depending on callback.
  """

  alias Membrane.Element.Pad
  alias Membrane.Core
  alias Core.Element.State
  use Bunch

  @macrocallback from_state(State.t(), keyword()) :: Macro.t()

  defmacro __using__(fields) do
    quote do
      default_fields_names = [:pads, :playback_state, :clock, :pipeline_clock]
      fields_names = unquote(fields |> Keyword.keys())

      @type t :: %__MODULE__{
              unquote_splicing(fields),
              pads: %{Pad.ref_t() => Pad.Data.t()},
              playback_state: Membrane.Element.playback_state_t()
            }

      @behaviour unquote(__MODULE__)

      @enforce_keys Module.get_attribute(__MODULE__, :enforce_keys)
                    ~> (&1 || fields_names)
                    |> Bunch.listify()
                    ~> (&1 ++ default_fields_names)

      defstruct fields_names ++ default_fields_names

      @impl true
      defmacro from_state(state, args \\ []) do
        quote do
          state = unquote(state)

          %unquote(__MODULE__){
            unquote_splicing(args),
            playback_state: state.playback.state,
            pads: state.pads.data,
            clock: state.synchronization.clock,
            pipeline_clock: state.synchronization.pipeline_clock
          }
        end
      end

      defoverridable unquote(__MODULE__)
    end
  end
end

==> membrane_core-0.4.2.tar: lib/membrane/element/callback_context.ex <==
defmodule Membrane.Element.CallbackContext do
  @moduledoc """
  Parent module for all contexts passed to callbacks.

  The idea of context is to provide access to commonly used information without
  holding it in elements state. Context differs depending on callback.
  """

  alias Membrane.Element.Pad
  alias Membrane.Core
  alias Core.Element.State
  use Bunch

  @macrocallback from_state(State.t(), keyword()) :: Macro.t()

  defmacro __using__(fields) do
    quote do
      default_fields_names = [:pads, :playback_state, :clock, :pipeline_clock]
      fields_names = unquote(fields |> Keyword.keys())

      @type t :: %__MODULE__{
              unquote_splicing(fields),
              pads: %{Pad.ref_t() => Pad.Data.t()},
              playback_state: Membrane.PlaybackState.t()
            }

      @behaviour unquote(__MODULE__)

      @enforce_keys Module.get_attribute(__MODULE__, :enforce_keys)
                    ~> (&1 || fields_names)
                    |> Bunch.listify()
                    ~> (&1 ++ default_fields_names)

      defstruct fields_names ++ default_fields_names

      @impl true
      defmacro from_state(state, args \\ []) do
        quote do
          state = unquote(state)

          %unquote(__MODULE__){
            unquote_splicing(args),
            playback_state: state.playback.state,
            pads: state.pads.data,
            clock: state.synchronization.clock,
            pipeline_clock: state.synchronization.pipeline_clock
          }
        end
      end

      defoverridable unquote(__MODULE__)
    end
  end
end
